// SPDX-License-Identifier: GPL-2.0-or-later

#define _JENT_AC(X,Y)   (X##Y)
#define JENT_AC(X,Y)            _JENT_AC(X,Y)

#define JENT_UL(x)              (JENT_AC(x, UL))
#define JENT_ULL(x)             (JENT_AC(x, ULL))

#define JENT_BIT(nr)            (JENT_UL(1) << (nr))
#define JENT_BIT_ULL(nr)        (JENT_ULL(1) << (nr))

#define JENT_CONF_DISABLE_LOOP_SHUFFLE

extern void jent_get_nstime(__u64 *out);
extern int jent_hash_time(void *hash_state, __u64 time, u8 *addtl,
			  unsigned int addtl_len, __u64 hash_loop_cnt,
			  unsigned int stuck);
int jent_read_random_block(void *hash_state, char *dst, unsigned int dst_len);

struct rand_data;
extern int jent_entropy_init(unsigned int osr, unsigned int flags,
			     void *hash_state, struct rand_data *p_ec);
extern int jent_read_entropy(struct rand_data *ec, unsigned char *data,
			     unsigned int len);

extern struct rand_data *jent_entropy_collector_alloc(unsigned int osr,
						      unsigned int flags,
						      void *hash_state);
extern void jent_entropy_collector_free(struct rand_data *entropy_collector);

#ifdef CONFIG_CRYPTO_JITTERENTROPY_TESTINTERFACE
int jent_raw_hires_entropy_store(__u32 value);
void jent_testing_init(void);
void jent_testing_exit(void);
#else /* CONFIG_CRYPTO_JITTERENTROPY_TESTINTERFACE */
static inline int jent_raw_hires_entropy_store(__u32 value) { return 0; }
static inline void jent_testing_init(void) { }
static inline void jent_testing_exit(void) { }
#endif /* CONFIG_CRYPTO_JITTERENTROPY_TESTINTERFACE */
